#ifndef H_MASS_OBJECT
#define H_MASS_OBJECT


#include "BasicObject.h"

class World;
#define INFINITE_DAMAGE 10e10
#define MAX_VELOCITY 500

/**
  each object which have mass is ancestor of this class
  */
class MassObject: public BasicObject{
    private:
        /**
          internal variable - change od position in actual tick
          */
        Point positionDelta;

        /**
          internal variable - change of velocity in actual tick
          */
        Point velocityDelta;

        /**
          internal variable - change of angle in actual tick
          */
        double angularDelta;

        /**
          internal variable - change of angular velocity in tick
          */
        double angularVelocityDelta;

        /**
          actual time qunatum, reseted by timeNotify()
          to obtain it, please use getTimeDelta()
          */
        double timeDelta;

    public:
        /**
          mass of object.
          standard values - 10e5 is BlackHole,
          10 is Ship,
          ?-3 Weapons
          5-10 asteroids
          */
        double mass;

        /**
          velocity of object,
          maximum veocity is 500 (due technical reasons),
          standard values:
          weapons 50-100
          ships <100
          */
        Point velocity;

        /**
          angular Mass
          standard value is 1/2 mass radius^2
          */
        double angularMass;

        /**
          angular velocity
          standard value is <3
          */
        double angularVelocity;

    public:
        /**
          constructor,
          set default values 
          */
        MassObject();

	virtual ~MassObject();


        /**
          at the end of the tick, set new object position
          */
        void flip();


        virtual void clean();
        virtual void timeNotify(const double timeDelta);

        /**
          tell object that he received damage
          damage from impact is Impactdamage
          damage from explosion is ExplosionDamage
          pointer to World is to provide capabilities of creating
          objects/explosions
          */
        virtual void doDamage(double Impactdamage, double
                ExplosionDamage, int from, World* world);

        /**
          called when object is destroyed. More precisely, when
          sweeping algorithm sweeps this object.
          pointer to World is provided to enable object/explosion
          creation
          */
        virtual void destroy(World* world); 


        /**
          serialize all MassObject's and BasicObject's data,
          without mark
          */
        static void _serialize(std::vector<unsigned char>&
                destination, MassObject* ptr);

        /**
          deserialize all MassObject's and BasicObject' data
          */
        static void _deserialize(std::vector<unsigned char>::iterator&
                it, MassObject* ptr);

        /**
          do Impact on object (without damage - each force should use
          this functions instead of accessing private data
          impact - change of position
          velocityImopact - change of velocity (note, that normally
          you shoud multiply force by timeDelta!
          angularImpact - change of angle
          angularVelocityImpact - change of angular speed (again,
          normally you multiply it by timeDelta!
          */
        void doImpact(const Point impact, const Point velocityImpact,
                      const double angularImpact, const double angularVelocityImpact);

        /**
          @see
          doImpact(impact,velocityImpact,angularImpact,angularVelocityImpact
          same as that, only impact=angularImpact=0
          */
        void doImpact(const Point velocityImpact, const double angularVelocityImpact);

        /**
          @see doImpact(velocityImpact,angularVelocityImpact)
          same as that, only angularVelocityImpact=0
          */
        void doImpact(const Point velocityImpact);

        /**
          return current time quantum asociated with object
          */
        double getTimeDelta();

        /**
          copy all MassObject's and BasicObject's data from one object
          to another. Used internally for cloning
          */
        static void copyData(const MassObject* from, MassObject* to);


        /**
          this function returns position of object at the end of the
          tick, but do not flip it, so you can use it to determine
          actual position of object without modifying data.
          */
        Point getFlipPosition();
};

// {{{ getTimeDelta
inline double MassObject::getTimeDelta(){
    return timeDelta;
}
// }}}

// {{{ getFlipPosition
inline Point MassObject::getFlipPosition(){
    return position+positionDelta+velocity*timeDelta;
}
// }}}


// {{{ doImpact(impact,velocityImpact,angularImpact,angularVelocityImpact)
inline void MassObject::doImpact(const Point impact, const Point velocityImpact,
         const double angularImpact, const double angularVelocityImpact){
    positionDelta+=impact;
    angularDelta+=angularImpact;
    velocityDelta+=velocityImpact;
    angularVelocityDelta+=angularVelocityImpact;
}
// }}}

// {{{ doImpact(velocityImpact, angularVelocityImpact)
inline void MassObject::doImpact(const Point velocityImpact, const double angularVelocityImpact){
    velocityDelta+=velocityImpact;
    angularVelocityDelta+=angularVelocityImpact;
}
// }}}

// {{{ doImpact(velocityImpact)
inline void MassObject::doImpact(const Point velocityImpact){
    velocityDelta+=velocityImpact;
}
// }}}


// {{{ timeNotify
inline void MassObject::timeNotify(const double timeDelta){
    this->timeDelta=timeDelta;
}
// }}}

// {{{ flip
inline void MassObject::flip(){
    position+=positionDelta+velocity*timeDelta;
    velocity+=velocityDelta;
    angle+=angularDelta+angularVelocity*timeDelta;
    angularVelocity+=angularVelocityDelta;
    if (velocity.length()>MAX_VELOCITY){
        /*
        fprintf(stderr,"%d %lf %lf, %lf %lf\n",
                getType(), position.x,position.y,
                velocity.x,velocity.y);        
        throw 1;
        */
        velocity/=velocity.length()/MAX_VELOCITY;
    }
}
// }}}

#endif
