#ifndef H_POINT
#define H_POINT

#include <cmath>
#include <algorithm>
#include <vector>
#include <string>

/**
  basic class representing vector in 2D space
  */
class Point{
    public:
        double x; /** x coordinate */
        double y; /** y coordinate */

        Point();  /** constructor, sets zero to x,y */

        /** constructor from doubles */
        Point(const double x,const double y);

        /** constructor from ints */
        Point(const int x,const int y);

        /** constructor of unitary vector */
        Point(double angle);

        /** unary minus */
        Point operator-();

        /** binary + */
        Point operator+(const Point p) const;

        /** binary - */
        Point operator-(const Point p) const;

        /** dot product */
        double operator*(const Point p) const;

        /** cross product */
        double operator^(const Point p) const;

        /** plus */
        Point& operator+=(const Point p);

        /** minus */
        Point& operator-=(const Point p);

        /** return length of vector */
        double length() const;

        /** return square of length */
        double length2() const;

        void multiplyBy(Point p);

        /** set vector's length to 1 */
        void normalize();

        /** normal multiplication by int */
        friend Point operator*(const int,const  Point);

        /** normal multiplication by double */
        friend Point operator*(const Point,const int);

        /** normal multiplication by double */
        friend Point operator*(const double,const Point);

        /** normal multiplication by int */
        friend Point operator*(const Point,const double);

        /** division by int */
        friend Point operator/(const Point,const int);

        /** division by double */
        friend Point operator/(const Point,const double);

        /** multiplication by int */
        Point& operator*=(int);

        /** multiplication by double */
        Point& operator*=(double);

        /** division by int */
        Point& operator/=(int);

        /** division by double */
        Point& operator/=(double);

        /** swap x and y */
        void flip();

        double angle() const;

        void serialize(std::vector<unsigned char>& destination);
        static Point deserialize(std::vector<unsigned char>::iterator&
                it);
        static Point getPointFromString(const std::string& str);
};

// {{{ flip
inline void Point::flip(){
    std::swap(x,y);
}
// }}}

// {{{ Point()
inline Point::Point():x(0),y(0){
}
// }}}

// {{{ Point(double,double)
inline Point::Point(const double x, const double y):x(x),y(y){
}
// }}}

// {{{ Point(angle)
inline Point::Point(double angle){
    x=cos(angle);
    y=sin(angle);
}
// }}}

// {{{ Point(int,int)
inline Point::Point(const int x,const int y):x(x),y(y){
}
// }}}

// {{{ operator-()
inline Point Point::operator-(){
    return Point(-x,-y);
}
// }}}

// {{{ operator+(Point)
inline Point Point::operator+(const Point p) const{
    return Point(p.x+x,p.y+y);
}
// }}}

// {{{ operator-(Point)
inline Point Point::operator-(const Point p) const{
    return Point(x-p.x,y-p.y);
}
// }}}

// {{{ operator*(Point)
inline double Point::operator*(const Point p) const{
    return (x*p.x+y*p.y);
}
// }}}

// {{{ operator^(Point)
inline double Point::operator^(const Point p) const{
    return x*p.y-y*p.x;
}
// }}}

// {{{ operator+=(Point)
inline Point& Point::operator+=(const Point p){
    x+=p.x;
    y+=p.y;
    return *this;
}
// }}}

// {{{ operator-=(Point)
inline Point& Point::operator-=(const Point p){
    x-=p.x;
    y-=p.y;
    return *this;
}
// }}}

// {{{ operator*(int,Point)
inline Point operator*(const int t,const Point p){
    return Point(p.x*t,p.y*t);
}
// }}}

// {{{ operator*(Point,int)
inline Point operator*(const Point p,const int t){
    return Point(p.x*t,p.y*t);
}
// }}}

// {{{ operator*(double,Point)
inline Point operator*(const double t,const Point p){
    return Point(p.x*t,p.y*t);
}
// }}}

// {{{ operator*(Point,double)
inline Point operator*(const Point p,const double t){
    return Point(p.x*t,p.y*t);
}
// }}}

// {{{ operator/(Point,int)
inline Point operator/(const Point p,const int t){
    return Point(p.x/t,p.y/t);
}
// }}}

// {{{ operator/(Point,double)
inline Point operator/(const Point p,const double t){
    return Point(p.x/t,p.y/t);
}
// }}}

// {{{ operator*=(int)
inline Point& Point::operator*=(const int t){
    x*=t;
    y*=t;
    return *this;
}
// }}}

// {{{ operator*=(double)
inline Point& Point::operator*=(const double t){
    x*=t;
    y*=t;
    return *this;
}
// }}}

// {{{ operator/=(int)
inline Point& Point::operator/=(const int t){
    x/=t;
    y/=t;
    return *this;
}
// }}}

// {{{ operator/=(double)
inline Point& Point::operator/=(const double t){
    x/=t;
    y/=t;
    return *this;
}
// }}}

// {{{ length()
inline double Point::length() const{
    return sqrt(x*x+y*y);
}
// }}}

// {{{ length2
inline double Point::length2() const{
    return x*x+y*y;
}
// }}}

// {{{
inline void Point::multiplyBy(Point p){
    x*=p.x;
    y*=p.y;
}

// }}}

inline double Point::angle() const {
    return atan2(y,x);
}
#endif
