#ifndef H_WORLD
#define H_WORLD

#include "MassObject.h"
#include "Ship.h"
#include <vector>
#include "Collider.h"
#include "EdgeCollider.h"
#include "Gravity.h"
#include "Preferences.h"
#include "Graphics.h"
#include "AsteroidSpawningSubsystem.h"
#include "Communication.h"
#include "Explosion.h"
#include "PlayerStatistics.h"
#include "WorldArrays.h"
#include "Nebula.h"
#define MAX_PLAYERS 5

class Ship;


class World{
    public:
    /**
      constructor
      */
    World();

    /**
      destructor
      */
    ~World();

    /**
      compute new world's state
      */
    void computeNextFrame(double timeDelta);

    std::string mapName;


    void receiveDataFromPlayers();
    void sendDataToViewer();
    void sendDataToPlayers();
    void sendDataToStats();

    void prepareDataForPlayer(int player, WorldArrays*
            arrays);

    void initializeServer();

    /**
      initialize world and subsystems
      */
    void initialize(const std::string& map);

    void deserialize(std::vector<unsigned char>::iterator& it);
    void serialize(std::vector<unsigned char>& destination);

    void draw();

    int getObjectCount();

    double getRemainingTime();

    Ship* addShip(int player, int ship);
    Explosion* addExplosion(Point position, double damagePerSecond);
    PlayerStatistics playerStatistics;
    void addLightObject(MassObject* ptr);
    void addAffector(Affector* ptr);

    /**
      dimensions od world
      */
    int width,height;
    private:

    void loadBlackHoles(Preferences& mapData);
    void loadNebulas(Preferences& mapData);
    void loadTwirlers(Preferences& mapData);
    void loadShips(Preferences& mapData);
    void loadAffectors(Preferences& mapData);

    void clearAllObjects();

    public:
    Communication communication;
    private:
    CommunicationGraph communicationGraph;
    std::vector<int> shipCount;

    /**
      collider helper
      */
    Collider collider;

    /**
      edge collider helper
      */
    EdgeCollider edgeCollider;

    /**
      gravity helper
      */
    Gravity gravity;

    /**
      subsystem for creating asteroids
      */
    AsteroidSpawningSubsystem asteroidSpawningSubsystem;



    double time;
    double roundTime;
    int frame;

    /**
      maximum time we allow to skip,
      if we supply greater time, it will
      be automatically sliced
      */
    double maxFrameTime;

    public:
    /**
      blackHoles
      */
    std::vector<MassObject*> heavyObjects;

    /**
      asteroids, weapons
      */
    std::vector<MassObject*> lightObjects;

    /**
      ships
      */
    std::vector<Ship*> ships;

    /**
      graphics helpers, affectors, bonuses
      */
    std::vector<BasicObject*> masslessObjects;

    void nextFrame(const double timeDelta);

    /**
      clean all MassObjects's internal state's
      */
    void clean();

    /**
      update ships from player response data
      */
    void computePlayerResponses(double timeDelta);

    /**
      notify all objects about timeDelta we are using in
      this frame
      */
    void timeNotify(double timeDelta);

    /**
      add gravitational acceleration from all heavyObjects
      */
    void computeGravitationalAcceleration();

    
    /**
      compute collisions of objects, do damage and spawning of new
      objects
      */
    void computeCollisions();

    /**
      force all MassObjects from World to stay at screen
      (uses width, heigth to determine actual screen dimensions,
      */
    void computeEdgeCollisions();


    /**
      performs actual changing of position and speed, new positions
      */
    void flip();

    /**
      delete all dead objects
      */
    void wipe();
};


#endif
