#include <cmath>

#include "Explosion.h"
#include "Preferences.h"
#include "MassObject.h"
#include "World.h"
#include "MyMath.h"
#include "Types.h"
#include "Serializer.h"
#include "Random.h"

#include "Errors.h"
#include "Assert.h"

double Explosion::expandSpeed;
double Explosion::minimalDamage;
double Explosion::impactFactor;
double Explosion::impactDamage;
double Explosion::decay;

// {{{ Explosion OK
Explosion::Explosion(){
    radius=1;
    damagePerSecond=1;
    timeDelta=0;
    velocity=Point(0,0);
    angle=Random::getDouble(2*M_PI);
}
// }}}

// {{{ ~Explosion OK
Explosion::~Explosion(){

}
// }}}


// {{{ getExplosionRange OK
double Explosion::getExplosionRange(double objectDistance, double
        objectRadius){
    double t=MyMath::sqr(objectDistance) +
             MyMath::sqr(radius) -
             MyMath::sqr(objectRadius);

    if (objectDistance<1) return radius>objectRadius?0:2*M_PI;   

    t=MyMath::sqr( t/ (2 * objectDistance * radius));
    
    if (t>=1) return 0.0;
    assert(t>=0,"ups...");

    return exp(log((2*asin(sqrt(1-t))))*0.7);
}
// }}}

// {{{ initialize OK
void Explosion::initialize(){
    expandSpeed=Preferences::_getDouble("/explosion/expandSpeed");
    impactFactor=Preferences::_getDouble("/explosion/impactFactor");
    impactDamage=Preferences::_getDouble("/explosion/impactDamage");
    minimalDamage=Preferences::_getDouble("/explosion/minimalDamage");
    decay=Preferences::_getDouble("/explosion/decay");
}
// }}}

// {{{ damagePerAngle
double Explosion::damagePerAngle(){
    return damagePerSecond*exp(log(radius)*decay);

}
// }}}

// {{{ timeNotify OK
void Explosion::timeNotify(double timeDelta){
    radius+=timeDelta*expandSpeed;
    if (damagePerAngle()/radius<minimalDamage) alive=false;
    this->timeDelta=timeDelta;
    position+=timeDelta*velocity;
}
// }}}


// {{{ affect OK
void Explosion::affect(MassObject* object, World* world){
  Point direction=object->position - this->position;

  double damage = damagePerAngle() * timeDelta *
        getExplosionRange(direction.length(), object->radius) / 2 /
        M_PI;
  if (damage==0) return;


  object->doDamage(impactDamage*damage,damage,owner,world);

  direction.normalize();
  assert(object->mass>0,"!(mass >0)");
  direction*=timeDelta*impactFactor*damage/object->mass;
  object->doImpact(direction);
}
// }}}

// {{{ getType OK
int Explosion::getType(){
    return Types::explosion;
}
// }}

// {{{ serialize OK
void Explosion::serialize(std::vector<unsigned char>& destination){
    Serializer::serializeChar(destination,getType());
    BasicObject::_serialize(destination,this);
    Serializer::serializeFloat(destination,damagePerSecond);
    Serializer::serializePoint(destination,velocity);
}
// }}}

// {{{ deserialize OK
Explosion* Explosion::deserialize(std::vector<unsigned char>::iterator& it){
    Explosion* ptr=new Explosion();
    BasicObject::_deserialize(it,ptr);
    ptr->damagePerSecond=Serializer::deserializeFloat(it);
    ptr->velocity=Serializer::deserializePoint(it);
    return ptr;
}
// }}}
